//*****************************************************************************
// File Name            : camera_if.v
//-----------------------------------------------------------------------------
// Function             : camera interface
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo@himwari_co 
//-----------------------------------------------------------------------------
// History
// -.-- 2017/7/25
// -.-- 201709/04
//*****************************************************************************
module camera_if (
  pclk,    
  c_vsync, 
  href,    
  data,    
  rstb,
  wea,
  addra,
  dina 
);

  input pclk;
  input c_vsync;
  input href;
  input [7:0]data;
  input rstb;
  output wea;
  output [19 : 0] addra;
  output [15 : 0] dina;
  reg c_vsync_d1;
  reg c_vsync_d2;
  reg c_vsync_d3;
  reg href_d1;
  reg href_d2;
  reg href_d3;
  reg [7:0]data_d1; 
  reg [7:0]data_d2; 
  reg [7:0]data_d3; 
  reg  [11:0]i_h_cnt;
  reg  [11:0]i_v_cnt;
  wire [19:0] addra_h_base;
  wire [19:0] addra_h;
  wire [19:0] addra_l_base;
  wire [19:0] addra_l;
  wire       wea;
  wire [15:0] dina;
  wire [4:0]  r_data;
  wire [5:0]  g_data;
  wire [4:0]  b_data;
  parameter  vram_size_v = 400;         //VRAM垂直方向サイズ
  parameter  vram_size_h = 600;         //VRAM横方向サイズ
  parameter  v_start = 12'd0;          //垂直方向切り出し開始位置
  parameter  v_end = v_start + (vram_size_v-1); //垂直方向切り出し終了位置
  parameter  h_start = 12'd0;          //水平方向切り出し開始位置
  parameter  h_start_p = h_start * 2;        //水平方向切り出し開始ポイント
  parameter  h_end_p = h_start_p + (2*vram_size_h -1); //水平方向切り出し終了位置
  parameter  out_start_v = 40;          //出力画像垂直方向開始位置
  parameter  out_start_h = 100;         //出力画像水平方向開始位置終了位置

//カメラデータのりタイミング
always @ (posedge pclk or negedge rstb )//sim
//always @ (posedge clk or negedge rstb )
if (rstb==1'b0)
  begin
    c_vsync_d1 <= 1'b0 ;
    c_vsync_d2 <= 1'b0 ;
    c_vsync_d3 <= 1'b0 ;
    href_d1 <= 1'b0 ;
    href_d2 <= 1'b0 ;
    href_d3 <= 1'b0 ;
    data_d1 <= 8'h0 ; 
    data_d2 <= 8'h0 ; 
    data_d3 <= 8'h0 ; 
  end
else
  begin
    c_vsync_d1 <= c_vsync ;
    c_vsync_d2 <= c_vsync_d1 ;
    c_vsync_d3 <= c_vsync_d2 ;
    href_d1 <= href ;
    href_d2 <= href_d1 ;
    href_d3 <= href_d2 ;
    data_d1 <= data ; 
    data_d2 <= data_d1 ; 
    data_d3 <= data_d2 ; 
  end

//カメラデータの垂直方向カウンタ、水平方向カウンタ
always @ (posedge pclk or negedge rstb )
if (rstb==1'b0)
  begin
     i_v_cnt <= 12'h000;
     i_h_cnt <= 12'h000;
  end
else
  if((c_vsync_d2==1'b1)&&(c_vsync_d3==1'b0)) //vsync
     begin
       i_v_cnt <= 12'h000;
       i_h_cnt <= 12'h000;
     end
  else
     if((href_d2==1'b0)&&(href_d3==1'b1)) //line end
       begin
         i_v_cnt <= i_v_cnt + 12'h001;
         i_h_cnt <= 12'h000;
       end
     else if (href_d2==1'b1) //data enable
       begin
         i_v_cnt <= i_v_cnt ;
         i_h_cnt <= i_h_cnt + 12'h001;
       end
     else
       begin
         i_v_cnt <= i_v_cnt ;
         i_h_cnt <= i_h_cnt ;
       end
   

 assign addra_h_base = (i_v_cnt - v_start + out_start_v )*800;  
 assign addra_h = ((i_v_cnt>=v_start)&&(i_v_cnt<=v_end))? addra_h_base:7'h00;
 assign addra_l_base = i_h_cnt - h_start_p;  
 assign addra_l = ((i_h_cnt>=h_start_p)&&(i_h_cnt<=h_end_p))?(addra_l_base[19:1]+out_start_h):20'h0;
 assign addra = addra_h+addra_l;
 
//VRAM書き込みイネーブル
 assign wea =(((i_v_cnt>=v_start)&&(i_v_cnt<=v_end))&&((i_h_cnt>=h_start_p)&&(i_h_cnt<=h_end_p)))? i_h_cnt[0]:1'b0;

//rgb565
assign r_data = data_d2[7:3]; 
assign g_data = {data_d2[2:0],data_d1[7:5]}; 
assign b_data = data_d1[4:0]; 

assign dina = {r_data,g_data,b_data};

endmodule








