//*****************************************************************************
// File Name            : camera_mux.v
//-----------------------------------------------------------------------------
// Function             : camera_mux 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo@himwari_co 
//-----------------------------------------------------------------------------
// History
// -.-- 2017/7/25
// -.-- 2017/9/02
// -.-- 2017/9/03
//*****************************************************************************
module camera_mux (
	CLK,
	RSTB,
	SIO_D,
	SIO_C,
   XCLK,PCLK,VSYNC,HREF,D,
   DCLK_M,
	DATA_M,
	VSD_M,HSD_M,DEN_M,
	LED,
	sdram_clk,
	sdram_addr,
	sdram_ba,
	sdram_cas_n,
	sdram_cke,
	sdram_cs_n,
	sdram_dq,
	sdram_dqm,
	sdram_ras_n,
	sdram_we_n
	);	

  input		CLK;
  input		RSTB;
  inout		SIO_C;
  inout		SIO_D;
  output XCLK;  //camera system clock
  input PCLK;
  input VSYNC;
  input HREF;
  input [9:2]D;
  output    DCLK_M;
  output    sdram_clk;
  output  [12:0]  sdram_addr;
  output  [1:0]  sdram_ba;
  output    sdram_cas_n;
  output    sdram_cke;
  output    sdram_cs_n;
  inout  [15:0]  sdram_dq;
  output  [1:0]  sdram_dqm;
  output    sdram_ras_n;
  output    sdram_we_n;
  output [3:0]DATA_M;
  output VSD_M;
  output HSD_M;  
  output DEN_M;
  output LED;
  //clks
  wire clk_48m;  
  wire clk_144m;
  wire dclk;  
  // reset
  wire reset;
  wire locked;
  wire rstb_locked;
  //i2c
  wire		sccb_sccb_sda_oe;
  wire		sccb_sccb_scl_oe;
  //varm
  wire  wea;
  wire [19 : 0] addra;
  wire [15 : 0] dina;
  //vout
  wire	[23:0]	vout_RGB_OUT;
  wire		vout_HD;
  wire		vout_VD;
  wire		vout_DEN;
  //vin
  wire   [15:0]odata;
  wire   [19:0]oadr;
  wire	[27:0]	vin_address;
  wire	[3:0]	vin_byte_enable;
  wire		vin_read;
  wire		vin_write;
  wire	[31:0]	vin_write_data;
  wire		vin_acknowledge;
  wire	[31:0]	vin_read_data;
  wire   [7:0] pio_0_export;
	
	assign reset =~RSTB;
	assign rstb_locked = ((RSTB==1'b0)||(locked==1'b0))?1'b0:1'b1;
	
	clk_altpll clk_altpll(
	.areset(reset),
	.inclk0(CLK),
	.c0(clk_48m),
	.c1(clk_144m),
	.c2(sdram_clk),
	.c3(XCLK),
	.c4(dclk),
	.locked(locked));
	
	nios2e_sys nios2e_sys(
		.clk_clk      (clk_144m),       //   clk.clk
		.reset_reset_n (rstb_locked), // reset.reset_n
		.sccb_sda_in   (SIO_D),   //  sccb.sda_in
		.sccb_scl_in   (SIO_C),   //      .scl_in
		.sccb_sda_oe   (sccb_sda_oe),   //      .sda_oe
		.sccb_scl_oe   (sccb_scl_oe),    //      .scl_oe
		.sdram_addr       (sdram_addr),       // sdram.addr
		.sdram_ba         (sdram_ba),         //      .ba
		.sdram_cas_n      (sdram_cas_n),      //      .cas_n
		.sdram_cke        (sdram_cke),        //      .cke
		.sdram_cs_n       (sdram_cs_n),       //      .cs_n
		.sdram_dq         (sdram_dq),         //      .dq
		.sdram_dqm        (sdram_dqm),        //      .dqm
		.sdram_ras_n      (sdram_ras_n),      //      .ras_n
		.sdram_we_n       (sdram_we_n),      //      .we_n
		.vout_clk_clk       (dclk),       //   vout_clk.clk
		.vout_reset_reset_n (rstb_locked),  // vout_reset.reset_n
		.vout_RGB_OUT (vout_RGB_OUT), // vout.RGB_OUT
		.vout_HD      (vout_HD),      //      .HD
		.vout_VD      (vout_VD),      //      .VD
		.vout_DEN     (vout_DEN),     //      .DEN
		.vin_address      (vin_address),      //   vin.address
		.vin_byte_enable  (vin_byte_enable),  //      .byte_enable
		.vin_read         (vin_read),         //      .read
		.vin_write        (vin_write),        //      .write
		.vin_write_data   (vin_write_data),   //      .write_data
		.vin_acknowledge  (vin_acknowledge),  //      .acknowledge
		.vin_read_data    (vin_read_data)     //      .read_data
	);
	
   assign SIO_D = (sccb_sda_oe==1'b1)?1'b0:1'bz;
   assign SIO_C = (sccb_scl_oe==1'b1)?1'b0:1'bz;

  camera_if camera_if(
  .pclk(     PCLK),                 
  .c_vsync(  VSYNC), 
  .href(     HREF),           
  .data(     D),    
  .rstb(     rstb_locked), 
  .wea(wea),     
  .addra(addra), 
  .dina( dina)
);

  vin_adp vin_adp(
  .iclk  (PCLK ),
  .idata (dina),
  .iadr  (addra ),
  .iwen  (wea ),
  .oclk  (clk_144m),
  .odata (odata),
  .oadr  (oadr ),
  .owen  (vin_write),
  .oack  (vin_acknowledge ),
  .rstb  (rstb_locked )
   );

	assign vin_address ={6'h00,oadr,2'b00};
	assign vin_write_data = {8'h00,odata[15:11],3'h0,odata[10:5],2'h0,odata[4:0],3'h0};
	assign vin_byte_enable = 4'b1111;
	
	
  rgb_mux rgb_mux(
  .clk(    clk_48m),
  .rstb(   rstb_locked),
  .hsync(  vout_HD),
  .vsync(  vout_VD),
  .r_da(   vout_RGB_OUT[7:3]),
  .g_da(   vout_RGB_OUT[15:10]),
  .b_da(   vout_RGB_OUT[23:19]),
  .den(    vout_DEN),
  .dclk_m( DCLK_M),
  .hsync_m(HSD_M),
  .vsync_m(VSD_M),
  .da_m( DATA_M),
  .den_m(  DEN_M)
  );
  
endmodule