//*****************************************************************************
// File Name            : camera_mux.v
//-----------------------------------------------------------------------------
// Function             : camera_mux 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo@himwari_co 
//-----------------------------------------------------------------------------
// History
// -.-- 2017/7/25
// -.-- 2017/9/02
// -.-- 2017/9/03
//*****************************************************************************
module camera_mux (
	CLK,
	RSTB,
	SIO_D,
	SIO_C,
   XCLK,PCLK,VSYNC,HREF,D,
   DCLK_M,
	DATA_M,
	VSD_M,HSD_M,DEN_M,
	LED,
	sdram_clk,
	sdram_addr,
	sdram_ba,
	sdram_cas_n,
	sdram_cke,
	sdram_cs_n,
	sdram_dq,
	sdram_dqm,
	sdram_ras_n,
	sdram_we_n
	);	

  input		CLK;
  input		RSTB;
  inout		SIO_C;
  inout		SIO_D;
  output XCLK;  //camera system clock
  input PCLK;
  input VSYNC;
  input HREF;
  input [9:2]D;
  output    DCLK_M;
  output    sdram_clk;
  output  [12:0]  sdram_addr;
  output  [1:0]  sdram_ba;
  output    sdram_cas_n;
  output    sdram_cke;
  output    sdram_cs_n;
  inout  [15:0]  sdram_dq;
  output  [1:0]  sdram_dqm;
  output    sdram_ras_n;
  output    sdram_we_n;
  /*
  output [1:0]da_r;
  output [1:0]da_g;
  output [1:0]da_b;
  */
  output [3:0]DATA_M;
  output VSD_M;
  output HSD_M;  
  output DEN_M;
  output LED;
  //clks
  wire clk_48m;  
  wire clk_144m;
  wire dclk;  
  // reset
  wire reset;
  wire locked;
  wire rstb_locked;
  //i2c
	wire		sccb_sccb_sda_oe;
	wire		sccb_sccb_scl_oe;
	//reg clk_reg;
	//reg [3:0]cnt;
	
  wire h_c_en;
  wire [9:0] v_c;
  wire [9:0] h_c;
  wire [7:0]da_r_tmp;
  wire [7:0]da_g_tmp;
  wire [7:0]da_b_tmp;
  wire da_en_tmp;
  wire hsync_tmp;
  wire vsync_tmp;
  //varm
  wire  wea;
  wire [13 : 0] addra;
  wire [15 : 0] dina;
  wire [13 : 0] addrb;
  wire [15 : 0] doutb;
  //vout
  wire in_a_da_en;
  wire [7:0]in_a_da_r;
  wire [7:0]in_a_da_g;
  wire [7:0]in_a_da_b;
  //
  wire	[23:0]	vout_RGB_OUT;
  wire		vout_HD;
  wire		vout_VD;
  wire		vout_DEN;
	
	assign reset =~RSTB;
	assign rstb_locked = ((RSTB==1'b0)||(locked==1'b0))?1'b0:1'b1;
	
	clk_altpll clk_altpll(
	.areset(reset),
	.inclk0(CLK),
	.c0(clk_48m),
	.c1(clk_144m),
	.c2(sdram_clk),
	.c3(XCLK),
	.c4(dclk),
	.locked(locked));
	
	nios2e_sys nios2e_sys(
		.clk_clk      (clk_144m),       //   clk.clk
		.reset_reset_n (rstb_locked), // reset.reset_n
		.sccb_sda_in   (SIO_D),   //  sccb.sda_in
		.sccb_scl_in   (SIO_C),   //      .scl_in
		.sccb_sda_oe   (sccb_sda_oe),   //      .sda_oe
		.sccb_scl_oe   (sccb_scl_oe),    //      .scl_oe
		.sdram_addr       (sdram_addr),       // sdram.addr
		.sdram_ba         (sdram_ba),         //      .ba
		.sdram_cas_n      (sdram_cas_n),      //      .cas_n
		.sdram_cke        (sdram_cke),        //      .cke
		.sdram_cs_n       (sdram_cs_n),       //      .cs_n
		.sdram_dq         (sdram_dq),         //      .dq
		.sdram_dqm        (sdram_dqm),        //      .dqm
		.sdram_ras_n      (sdram_ras_n),      //      .ras_n
		.sdram_we_n       (sdram_we_n),      //      .we_n
		.vout_clk_clk       (dclk),       //   vout_clk.clk
		.vout_reset_reset_n (rstb_locked),  // vout_reset.reset_n
		.vout_RGB_OUT (vout_RGB_OUT), // vout.RGB_OUT
		.vout_HD      (vout_HD),      //      .HD
		.vout_VD      (vout_VD),      //      .VD
		.vout_DEN     (vout_DEN)      //      .DEN
	);
	
   assign SIO_D = (sccb_sda_oe==1'b1)?1'b0:1'bz;
   assign SIO_C = (sccb_scl_oe==1'b1)?1'b0:1'bz;

camera_if camera_if(
  .pclk(     rstb_locked),           
  .c_vsync(  VSYNC), 
  .href(     HREF),           
  .data(     D),    
  .dclk(     dclk),        
  .rstb(     RSTB), 
  .h_c_en(h_c_en),  
  .v_c(v_c),
  .h_c(h_c),             
  .gen_da_en(in_a_da_en),
  .gen_da_r( in_a_da_r), 
  .gen_da_g( in_a_da_g), 
  .gen_da_b( in_a_da_b), 
  .wea(wea),     
  .addra(addra), 
  .dina( dina),  
  .addrb(addrb), 
  .doutb(doutb) 
);


  rgb_mux rgb_mux(
  .clk(    clk_48m),
  .rstb(   rstb_locked),
  .hsync(  vout_HD),
  .vsync(  vout_VD),
  .r_da(   vout_RGB_OUT[7:3]),
  .g_da(   vout_RGB_OUT[15:10]),
  .b_da(   vout_RGB_OUT[23:19]),
  .den(    vout_DEN),
  .dclk_m( DCLK_M),
  .hsync_m(HSD_M),
  .vsync_m(VSD_M),
  .da_m( DATA_M),
  .den_m(  DEN_M)
  );
  
endmodule